/*
 * Copyright 2014-2016 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.fortress.randomizer.Randomizer;

import java.math.BigInteger;

final class GeneratorRandom extends DataGeneratorBase {
  public static final String NAME = "random";

  public GeneratorRandom() {
    super(NAME);
  }

  @Override
  protected void generate(final DataGeneratorQuery query) {
    final BigInteger min = query.hasParameter("min") ? query.getIntegerParameter("min") : null;
    final BigInteger max = query.hasParameter("max") ? query.getIntegerParameter("max") : null;

    final int size = query.hasParameter("size")
        ? query.getIntegerParameter("size").intValue() : -1;

    final boolean signExtend =
        query.getBooleanParameter("sign_extend");

    for (final DataGeneratorVariable variable : query.getUnknownVariables()) {
      final BitVector value = size == -1
          ? newRandomData(variable.getSize(), min, max)
          : newRandomData(size, min, max).resize(variable.getSize(), signExtend);
      setValue(variable, value);
    }
  }

  private static BitVector newRandomData(
      final int bitSize,
      final BigInteger min,
      final BigInteger max) {
    final BitVector result;

    if (min != null && max != null) {
      final BigInteger bi = Randomizer.get().nextBigIntegerRange(min, max);
      result = BitVector.valueOf(bi, bitSize);
    } else {
      result = BitVector.newEmpty(bitSize);
      Randomizer.get().fill(result);
    }

    return result;
  }
}
