/*
 * Copyright 2014-2015 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import org.junit.Assert;
import org.junit.Test;

import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.testbase.TestBaseQuery;
import ru.ispras.testbase.TestBaseQueryBuilder;
import ru.ispras.testbase.TestData;
import ru.ispras.testbase.knowledge.iterator.Iterator;

/**
 * {@link DataGeneratorTestCase} creates and executes a TestBase query,
 * which results in iterator a over a collection of test data.
 * This test is needed to see whether the combination logic works as expected.
 *
 * @author <a href="mailto:andrewt@ispras.ru">Andrei Tatarnikov</a>
 */
public final class DataGeneratorTestCase {

  private static TestBaseQuery newTestBaseQuery() {
    final TestBaseQueryBuilder builder = new TestBaseQueryBuilder();

    for (int size = 8; size <= 32; size *= 2) {
      final String valueName = String.format("val_bv%d", size);
      final String variableName = String.format("var_bv%d", size);

      builder.setBinding(valueName, NodeValue.newBitVector(size, size));
      builder.setBinding(variableName, NodeVariable.newBitVector(variableName, size));
    }

    return builder.build();
  }

  @Test
  public void testBoundaryValues() {
    final DataGenerator dataGenerator = new GeneratorBoundaryValues();
    final Iterator<TestData> iterator = dataGenerator.generate(newTestBaseQuery());

    int count = 0;
    for (iterator.init(); iterator.hasValue(); iterator.next()) {
      //System.out.println(iterator.value().toString());
      count++;
    }

    // To detect changes in the implementation.
    Assert.assertEquals(720, count);
  }
}
